@@ -4,16 +4,17 @@ from functools import partial, wraps
 from typing import Any, Callable, Type, Union
 
 from .utils import IgnoreType
-from .core import argname2, varname
+from .core import argname, varname
 
 
 def register(
     cls_or_func: type = None,
-    *,
+    # *, keyword-only argument, only available with python3.8+
     frame: int = 1,
     ignore: IgnoreType = None,
     multi_vars: bool = False,
     raise_exc: bool = True,
+    strict: bool = True,
 ) -> Union[Type, Callable]:
     """A decorator to register __varname__ to a class or function
 
@@ -31,6 +32,8 @@ def register(
             `VarnameRetrievingError` will be raised.
         raise_exc: Whether we should raise an exception if failed
             to retrieve the name.
+        strict: Whether to only return the variable name if the result of
+            the call is assigned to it directly.
 
     Examples:
         >>> @varname.register
@@ -58,6 +61,7 @@ def register(
                 ignore=ignore,
                 multi_vars=multi_vars,
                 raise_exc=raise_exc,
+                strict=strict,
             )
             orig_init(self, *args, **kwargs)
 
@@ -74,6 +78,7 @@ def register(
                 ignore=ignore,
                 multi_vars=multi_vars,
                 raise_exc=raise_exc,
+                strict=strict,
             )
 
             try:
@@ -90,6 +95,7 @@ def register(
         ignore=ignore,
         multi_vars=multi_vars,
         raise_exc=raise_exc,
+        strict=strict,
     )
 
 
@@ -109,6 +115,8 @@ class Wrapper:
     Args:
         value: The value to be wrapped
         raise_exc: Whether to raise exception when varname is failed to retrieve
+        strict: Whether to only return the variable name if the wrapper is
+            assigned to it directly.
 
     Attributes:
         name: The variable name to which the instance is assigned
@@ -121,9 +129,15 @@ class Wrapper:
         frame: int = 1,
         ignore: IgnoreType = None,
         raise_exc: bool = True,
+        strict: bool = True,
     ):
         # This call is ignored, since it's inside varname
-        self.name = varname(frame - 1, ignore=ignore, raise_exc=raise_exc)
+        self.name = varname(
+            frame=frame - 1,
+            ignore=ignore,
+            raise_exc=raise_exc,
+            strict=strict,
+        )
         self.value = value
 
     def __str__(self) -> str:
@@ -165,12 +179,12 @@ def debug(
         sep: The separator between the variable name and value
         repr: Print the value as `repr(var)`? otherwise `str(var)`
     """
-    var_names = argname2("var", "*more_vars", vars_only=vars_only, func=debug)
+    var_names = argname("var", "*more_vars", vars_only=vars_only, func=debug)
 
     values = (var, *more_vars)
     name_and_values = [
         f"{var_name}{sep}{value!r}" if repr else f"{var_name}{sep}{value}"
-        for var_name, value in zip(var_names, values) # type: ignore
+        for var_name, value in zip(var_names, values)  # type: ignore
     ]
 
     if merge:
