@@ -42,6 +42,13 @@ ArgSourceType = Union[ast.AST, str]
 ArgSourceType = Union[ArgSourceType, Tuple[ArgSourceType, ...]]
 ArgSourceType = Union[ArgSourceType, Mapping[str, ArgSourceType]]
 
+if sys.version_info >= (3, 8):
+    ASSIGN_TYPES = (ast.Assign, ast.AnnAssign, ast.NamedExpr)
+    AssignType = Union[ASSIGN_TYPES]  # type: ignore
+else:  # pragma: no cover
+    ASSIGN_TYPES = (ast.Assign, ast.AnnAssign)
+    AssignType = Union[ASSIGN_TYPES]  # type: ignore
+
 MODULE_IGNORE_ID_NAME = "__varname_ignore_id__"
 
 
@@ -55,32 +62,38 @@ class config:  # pylint: disable=invalid-name
     debug = False
 
 
-class VarnameRetrievingError(Exception):
+class VarnameException(Exception):
+    """Root exception for all varname exceptions"""
+
+
+class VarnameRetrievingError(VarnameException):
     """When failed to retrieve the varname"""
 
 
-class QualnameNonUniqueError(Exception):
+class QualnameNonUniqueError(VarnameException):
     """When a qualified name is used as an ignore element but references to
     multiple objects in a module"""
 
 
-class NonVariableArgumentError(Exception):
-    """When vars_only is True but try to retrieve name of
-    a non-variable argument"""
-
-
-class ImproperUseError(Exception):
+class ImproperUseError(VarnameException):
     """When varname() is improperly used"""
 
 
-class MaybeDecoratedFunctionWarning(Warning):
+class VarnameWarning(Warning):
+    """Root warning for all varname warnings"""
+
+
+class MaybeDecoratedFunctionWarning(VarnameWarning):
     """When a suspecious decorated function used as ignore function directly"""
 
 
-class MultiTargetAssignmentWarning(Warning):
+class MultiTargetAssignmentWarning(VarnameWarning):
     """When varname tries to retrieve variable name in
     a multi-target assignment"""
 
+class UsingExecWarning(VarnameWarning):
+    """When exec is used to retrieve function name for `argname()`"""
+
 
 @lru_cache()
 def cached_getmodule(codeobj: CodeType):
@@ -113,13 +126,14 @@ def get_node(
     return get_node_by_frame(frameobj, raise_exc)
 
 
-def get_node_by_frame(
-    frame: FrameType, raise_exc: bool = True
-) -> ast.AST:
+def get_node_by_frame(frame: FrameType, raise_exc: bool = True) -> ast.AST:
     """Get the node by frame, raise errors if possible"""
     exect = Source.executing(frame)
 
     if exect.node:
+        # attach the frame for better exception message
+        # (ie. where ImproperUseError happens)
+        exect.node.__frame__ = frame
         return exect.node
 
     if exect.source.text and exect.source.tree and raise_exc:
@@ -132,13 +146,16 @@ def get_node_by_frame(
     return None
 
 
-def lookfor_parent_assign(node: ast.AST) -> Union[ast.Assign, ast.AnnAssign]:
+def lookfor_parent_assign(node: ast.AST, strict: bool = True) -> AssignType:
     """Look for an ast.Assign node in the parents"""
     while hasattr(node, "parent"):
         node = node.parent
 
-        if isinstance(node, (ast.AnnAssign, ast.Assign)):
+        if isinstance(node, ASSIGN_TYPES):
             return node
+
+        if strict:
+            break
     return None
 
 
@@ -247,10 +264,7 @@ def check_qualname_by_source(
         )
 
 
-def debug_ignore_frame(
-    msg: str,
-    frameinfo: inspect.FrameInfo = None
-) -> None:
+def debug_ignore_frame(msg: str, frameinfo: inspect.FrameInfo = None) -> None:
     """Print the debug message for a given frame info object
 
     Args:
@@ -301,7 +315,6 @@ def get_argument_sources(
     node: ast.Call,
     func: Callable,
     vars_only: bool,
-    pos_only: bool,
 ) -> Mapping[str, ArgSourceType]:
     """Get the sources for argument from an ast.Call node
 
@@ -318,17 +331,14 @@ def get_argument_sources(
     # func(y, x, c=z)
     # ['y', 'x'], {'c': 'z'}
     arg_sources = [
-        argnode_source(source, argnode, vars_only) for argnode in node.args
+        argnode_source(source, argnode, vars_only)
+        for argnode in node.args
     ]
-    kwarg_sources = (
-        {
-            argnode.arg: argnode_source(source, argnode.value, vars_only)
-            for argnode in node.keywords
-            if argnode.arg is not None
-        }
-        if not pos_only
-        else {}
-    )
+    kwarg_sources = {
+        argnode.arg: argnode_source(source, argnode.value, vars_only)
+        for argnode in node.keywords
+        if argnode.arg is not None
+    }
     bound_args = signature.bind_partial(*arg_sources, **kwarg_sources)
     argument_sources = bound_args.arguments
     # see if *args and **kwargs have anything assigned
@@ -383,7 +393,8 @@ def get_function_called_argname(frame: FrameType, node: ast.AST) -> Callable:
         f"{pure_eval_fail_msg} "
         "Using 'eval' to get the function that calls 'argname'. "
         "Try calling it using a variable reference to the function, or "
-        "passing the function to 'argname' explicitly."
+        "passing the function to 'argname' explicitly.",
+        UsingExecWarning
     )
     expr = ast.Expression(node.func)
     code = compile(expr, "<ast-call>", "eval")
@@ -391,30 +402,38 @@ def get_function_called_argname(frame: FrameType, node: ast.AST) -> Callable:
     return eval(code, frame.f_globals, frame.f_locals)
 
 
-def parse_argname_subscript(node: ast.Subscript):
-    """Parse the ast.Subscript node passed to argname
+def rich_exc_message(msg: str, node: ast.AST, context_lines: int = 4) -> str:
+    """Attach the source code from the node to message to
+    get a rich message for exceptions
 
-    Make sure:
-    1. node.value is an ast.Name node
-    2. node.slice is a ast.Constant node
-
-    This is separated as a function, since we want to see in the future whether
-    non-ast.Constant node can be supported as node.slice.
+    If package 'rich' is not install or 'node.__frame__' doesn't exist, fall
+    to plain message (with basic information), otherwise show a better message
+    with full information
     """
-    name = node.value
-    if not isinstance(name, ast.Name):
-        raise ValueError(f"Expect {ast.dump(name)} to be a variable.")
+    frame = node.__frame__  # type: FrameType
+    lineno = node.lineno - 1  # type: int
+    col_offset = node.col_offset  # type: int
+    filename = frame.f_code.co_filename  # type: str
+    lines, startlineno = inspect.getsourcelines(frame)
+    startlineno = 0 if startlineno == 0 else startlineno - 1
+    line_range = (startlineno + 1, startlineno + len(lines) + 1)
 
-    subscript = node.slice # type: ast.AST
-    if isinstance(subscript, ast.Index):
-        subscript = subscript.value
-    if not isinstance(subscript, (ast.Str, ast.Num, ast.Constant)):
-        raise ValueError(f"Expect {ast.dump(subscript)} to be a constant.")
+    linenos = tuple(map(str, range(*line_range)))  # type: Tuple[str, ...]
+    lineno_width = max(map(len, linenos))  # type: int
+    hiline = lineno - startlineno  # type: int
+    codes = []  # type: List[str]
+    for i, lno in enumerate(linenos):
+        if i < hiline - context_lines or i > hiline + context_lines:
+            continue
+        lno = lno.ljust(lineno_width)
+        if i == hiline:
+            codes.append(f"  > | {lno}  {lines[i]}")
+            codes.append(f"    | {' ' * (lineno_width + col_offset + 2)}^\n")
+        else:
+            codes.append(f"    | {lno}  {lines[i]}")
 
-    subscript = getattr(
-        subscript, "value", subscript
-    )  # ast.Index, ast.Constant
-    subscript = getattr(subscript, "n", subscript)  # ast.Num
-    subscript = getattr(subscript, "s", subscript)  # ast.Str
-
-    return name.id, subscript
+    return (
+        f"{msg}\n\n"
+        f"  {filename}:{lineno+1}:{col_offset+1}\n"
+        f"{''.join(codes)}\n"
+    )
